import psycopg2
import psycopg2.extras # For dictionary cursor
import logging
from .db_core import get_db_connection, get_domain # Import from new core module

# Configure basic logging for this module
logger = logging.getLogger(__name__)

def add_favorite(user_id: int, article_id: int):
    """Adds an article to a user's favorites."""
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("INSERT INTO user_favorites (user_id, article_id) VALUES (%s, %s) ON CONFLICT (user_id, article_id) DO NOTHING", (user_id, article_id))
            return True
    except Exception as e:
        logger.error(f"Failed to add favorite for user {user_id}, article {article_id}: {e}", exc_info=True)
        return False

def remove_favorite(user_id: int, article_id: int):
    """Removes an article from a user's favorites."""
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("DELETE FROM user_favorites WHERE user_id = %s AND article_id = %s", (user_id, article_id))
            return True
    except Exception as e:
        logger.error(f"Failed to remove favorite for user {user_id}, article {article_id}: {e}", exc_info=True)
        return False

def get_user_favorites_ids(user_id: int) -> set[int]:
    """Gets a set of article IDs favorited by the user."""
    ids = set()
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("SELECT article_id FROM user_favorites WHERE user_id = %s", (user_id,))
            rows = cursor.fetchall()
            ids = {row['article_id'] for row in rows}
    except Exception as e:
        logger.error(f"Failed to get favorite IDs for user {user_id}: {e}", exc_info=True)
    return ids

def get_user_favorite_articles(user_id: int) -> list[dict]:
    """Gets the full article details for a user's favorites."""
    articles = []
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("""
                SELECT fa.id, fa.title, fa.summary, fa.source_url, fa.tags, fa.created_timestamp, fa.image_filename
                FROM final_articles fa
                JOIN user_favorites uf ON fa.id = uf.article_id
                WHERE uf.user_id = %s
                ORDER BY fa.created_timestamp DESC
            """, (user_id,))
            rows = cursor.fetchall()
            articles = [dict(row) for row in rows]
    except Exception as e:
        logger.error(f"Failed to get favorite articles for user {user_id}: {e}", exc_info=True)
    return articles

def add_deleted_article(user_id: int, article_id: int):
    """Marks an article as deleted from a user's feed."""
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("INSERT INTO user_deleted_articles (user_id, article_id) VALUES (%s, %s) ON CONFLICT (user_id, article_id) DO NOTHING", (user_id, article_id))
            return True
    except Exception as e:
        logger.error(f"Failed to add deleted article for user {user_id}, article {article_id}: {e}", exc_info=True)
        return False

def get_deleted_article_ids(user_id: int) -> set[int]:
    """Gets a set of article IDs deleted by the user."""
    ids = set()
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("SELECT article_id FROM user_deleted_articles WHERE user_id = %s", (user_id,))
            rows = cursor.fetchall()
            ids = {row['article_id'] for row in rows}
    except Exception as e:
        logger.error(f"Failed to get deleted article IDs for user {user_id}: {e}", exc_info=True)
    return ids

def add_blacklisted_source(user_id: int, domain_name: str):
    """Adds a domain name to the user's blacklist."""
    clean_domain = get_domain(domain_name) # Ensure it's just the domain
    if not clean_domain:
        logger.warning(f"Attempted to blacklist invalid domain '{domain_name}' for user {user_id}")
        return False
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("INSERT INTO user_source_blacklist (user_id, domain_name) VALUES (%s, %s) ON CONFLICT (user_id, domain_name) DO NOTHING", (user_id, clean_domain))
            logger.info(f"User {user_id} blacklisted domain: {clean_domain}")
            return True
    except Exception as e:
        logger.error(f"Failed to add blacklisted domain '{clean_domain}' for user {user_id}: {e}", exc_info=True)
        return False

def remove_blacklisted_source(user_id: int, domain_name: str):
    """Removes a domain name from the user's blacklist."""
    clean_domain = get_domain(domain_name) # Ensure consistency
    if not clean_domain: return False
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("DELETE FROM user_source_blacklist WHERE user_id = %s AND domain_name = %s", (user_id, clean_domain))
            logger.info(f"User {user_id} removed domain from blacklist: {clean_domain}")
            return True
    except Exception as e:
        logger.error(f"Failed to remove blacklisted domain '{clean_domain}' for user {user_id}: {e}", exc_info=True)
        return False

def get_blacklisted_sources(user_id: int) -> set[str]:
    """Gets a set of domain names blacklisted by the user."""
    domains = set()
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("SELECT domain_name FROM user_source_blacklist WHERE user_id = %s", (user_id,))
            rows = cursor.fetchall()
            domains = {row['domain_name'] for row in rows}
    except Exception as e:
        logger.error(f"Failed to get blacklisted sources for user {user_id}: {e}", exc_info=True)
    return domains

def get_article_source_url(article_id: int) -> str | None:
    """Gets the source URL for a given article ID."""
    url = None
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("SELECT source_url FROM final_articles WHERE id = %s", (article_id,))
            row = cursor.fetchone()
            if row:
                url = row['source_url']
    except Exception as e:
        logger.error(f"Failed to get source URL for article {article_id}: {e}", exc_info=True)
    return url
